@@ -25,6 +25,11 @@
 #define __ABSTRACT_IO_H_INCLUDED
 
 #include "objects.h"
+#include "objtypes.h"
+
+#if HAVE_SYS_SOCKET_H
+#include <sys/socket.h>
+#endif
 
 typedef void abstract_addr;
 
@@ -47,7 +52,7 @@ typedef void abstract_addr;
      (name abstract_read)
      (vars
        (read indirect-method int "UINT32 length" "UINT8 *buffer")
-       (recv indirect-method int "UINT32 length" "UINT8 *buffer" "abstract_addr *addr" "size_t *addrsize")))
+       (recv indirect-method int "UINT32 length" "UINT8 *buffer" "abstract_addr *addr" "socklen_t *addrsize")))
 */
 
 #define A_READ(f, length, buffer) (f)->read(&(f), (length), (buffer))
