@@ -211,8 +211,8 @@ void gc_maybe(struct ol_object *root, int busy)
   sanity_check_object_list();
 #endif
 
-  if (busy && (number_of_objects > gc_busy_threshold) ||
-      !busy && (number_of_objects > gc_idle_threshold))
+  if ((busy && (number_of_objects > gc_busy_threshold)) ||
+      (!busy && (number_of_objects > gc_idle_threshold)))
     { 
 
       verbose("Garbage collecting while %z...\n", busy ? "busy" : "idle");
