@@ -131,9 +131,9 @@ closes
 					 * to choose an exit code. */
 					if (fd->close_callback && fd->close_reason)
 						(void) CLOSE_CALLBACK(fd->close_callback, fd->close_reason);
-#if 0					
+
 					debug("Closing fd %i.\n", fd->fd);
-#endif
+
 					close(fd->fd);
 				}
 				/* Unlink this fd */
@@ -317,9 +317,7 @@ static int do_read(struct abstract_read **r, UINT32 length, UINT8 *buffer)
 #endif
       if (!res)
 	{
-#if 0
 	  debug("Read EOF on fd %i.\n", closure->fd);
-#endif
 	  return A_EOF;
 	}
       if (res > 0)
@@ -338,16 +336,16 @@ static int do_read(struct abstract_read **r, UINT32 length, UINT8 *buffer)
 	default:
 	  werror("io.c: do_read: read() failed (errno %i), %z\n",
 		 errno, strerror(errno));
-#if 0
+
 	  debug("  fd = %i, buffer = %xi, length = %i\n",
 		closure->fd, buffer, length);
-#endif
+
 	  return A_FAIL;
 	}
     }
 }
 
-static int do_recv(struct abstract_read **r, UINT32 length, UINT8 *buffer, abstract_addr *addr, size_t *addrlen)
+static int do_recv(struct abstract_read **r, UINT32 length, UINT8 *buffer, abstract_addr *addr, socklen_t *addrlen)
 {
 	CAST(fd_read, closure, *r);
 
@@ -359,9 +357,9 @@ static int do_recv(struct abstract_read **r, UINT32 length, UINT8 *buffer, abstr
 	for (;;) {
 		int res = recvfrom(closure->fd, buffer, length, 0, (struct sockaddr *) addr, addrlen);
 		if (!res) {
-#if 0
+
 			debug("Read EOF on fd %i.\n", closure->fd);
-#endif
+
 			return A_EOF;
 		}
 		if (res > 0)
@@ -379,10 +377,10 @@ static int do_recv(struct abstract_read **r, UINT32 length, UINT8 *buffer, abstr
 		default:
 			werror("io.c: do_recv: recvfrom() failed (errno %i), %z\n",
 			       errno, strerror(errno));
-#if 0
+
 			debug("  fd = %i, buffer = %xi, length = %i\n",
 			      closure->fd, buffer, length);
-#endif
+
 			return A_FAIL;
 		}
 	}
@@ -481,9 +479,11 @@ static void write_callback(struct nonblocking_fd *fd)
 	assert(self->buffer);
 
 	res = BUF_FLUSH(self->buffer, &w.super);
-	if (ST_CLOSEDP(res))
+	if (res & ST_CLOSE)
 	{
 		close_fd(fd, ST_FAILUREP(res) ? CLOSE_PROTOCOL_FAILURE : CLOSE_EOF);
+	} else if (res & ST_DIE) {
+		closekill_fd(fd, ST_FAILUREP(res) ? CLOSE_PROTOCOL_FAILURE : CLOSE_EOF);
 	}
 }  
 
@@ -492,7 +492,7 @@ static void listen_callback(struct nonblocking_fd *fd)
 	CAST(listen_fd, self, fd);
 	char sockaddr_buf[256];
 	struct sockaddr *peer = (struct sockaddr *) &sockaddr_buf;
-	size_t addr_len = sizeof(sockaddr_buf);
+	socklen_t addr_len = sizeof(sockaddr_buf);
 	int res;
 	int conn;
 
@@ -522,7 +522,7 @@ static void connect_callback(struct nonblocking_fd *fd)
 {
 	CAST(connect_fd, self, fd);
 	int sock_err;
-	size_t len = sizeof(sock_err);
+	socklen_t len = sizeof(sock_err);
   
   	if ((getsockopt(fd->fd, SOL_SOCKET, SO_ERROR, (char *) &sock_err, &len) < 0) ||
   	    sock_err) {
@@ -824,10 +824,12 @@ static int inet_address2sockaddr_in(struct address_info *c, int sasize, struct s
 
 static int bind_inet_socket(struct address_info *c, int fd)
 {
+	CAST(inet_address_info, i, c);
 	struct sockaddr_in in;
 	int tmp = 1;
 
 	if (inet_address2sockaddr_in(c, sizeof(in), (struct sockaddr *) &in)) {
+		verbose("binding fd %i, inetaddr: %S, port: %i\n", fd, i->ip, i->port);
 		setsockopt(fd, SOL_SOCKET, SO_REUSEADDR, &tmp, sizeof(tmp));
 		if (bind(fd, (struct sockaddr *) &in, sizeof(struct sockaddr_in)) == -1) {
 			werror("io.c: bind_inet_socket() bind failed %I:%i %z\n", in.sin_addr, ntohs(in.sin_port), strerror(errno));
@@ -839,9 +841,11 @@ static int bind_inet_socket(struct address_info *c, int fd)
 
 static int connect_inet_socket(struct address_info *c, int fd)
 {
+	CAST(inet_address_info, i, c);
 	struct sockaddr_in in;
 
 	if (inet_address2sockaddr_in(c, sizeof(in), (struct sockaddr *) &in)) {
+		verbose("connecting fd %i to inetaddr %S, port %i\n", fd, i->ip, i->port);
 		if (connect(fd, (struct sockaddr *) &in, sizeof(in)) < -1) {
 			switch (errno) {
 			case EWOULDBLOCK:
@@ -1137,9 +1141,9 @@ struct connect_fd *io_connect(struct io_backend *b,
 			      struct address_info *remote,
 			      struct fd_callback *f)
 {
-#if 0
+
 	debug("io.c: connecting using fd %i\n", s);
-#endif
+
 	if (!ADDRESS_CONNECT(remote, s)) {
 		int saved_errno = errno;
 		close(s);
@@ -1165,9 +1169,9 @@ struct listen_fd *io_listen(struct io_backend *b,
 			    struct fd_listen_callback *callback)
 {
 	if (s < 0) return NULL;
-#if 0
+
 	debug("io.c: listening on fd %i\n", s);
-#endif
+
 
 	if (listen(s, 256) < 0) {
 		close(s);
@@ -1203,9 +1207,9 @@ struct io_fd *io_read_write(struct io_fd *f,
 			    struct abstract_buffer *buffer,
 			    struct close_callback *close_callback)
 {
-#if 0
-	debug("io.c: Preparing fd %i for reading and writing\n", fd);
-#endif
+
+	debug("io.c: Preparing fd %i for reading and writing\n", f->super.fd);
+
 
 	f->super.prepare = prepare;
 
@@ -1229,9 +1233,9 @@ struct io_fd *io_read(struct io_fd *f,
 		      struct read_handler *handler,
 		      struct close_callback *close_callback)
 {
-#if 0
-	debug("io.c: Preparing fd %i for reading\n", fd);
-#endif
+
+	debug("io.c: Preparing fd %i for reading\n", f->super.fd);
+
 	f->super.prepare = prepare;
 
 	/* Reading */
@@ -1249,9 +1253,9 @@ struct io_fd *io_write(struct io_fd *f,
                        struct close_callback *close_callback)
 {
 
-#if 0
-	debug("io.c: Preparing fd %i for writing\n", fd);
-#endif
+
+	debug("io.c: Preparing fd %i for writing\n", f->super.fd);
+
 	f->super.prepare = prepare;
 
 	/* Writing */
@@ -1287,9 +1291,9 @@ void kill_fd(struct nonblocking_fd *fd)
 
 void close_fd(struct nonblocking_fd *fd, int reason)
 {
-#if 0
+
 	debug("Marking fd %i for closing.\n", fd->fd);
-#endif
+
 	fd->close_reason = reason;
 	if (!fd->to_be_closed)
 		fd->to_be_closed = 1;
